import { __awaiter, __decorate } from "tslib";
import { Connect } from '@genesislcap/foundation-comms';
import { CriteriaBuilder, Join } from '@genesislcap/foundation-criteria';
import { createLogger } from '@genesislcap/foundation-utils';
import { isControl, isStringControl, mapStateToControlProps, rankWith, } from '@jsonforms/core';
import { attr, customElement, observable } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { BooleanControlEntry } from '../jsonforms/renderers/BooleanControlRenderer';
import { ConnectedMultiselectControlRendererEntry } from '../jsonforms/renderers/ConnectedMultiselectControlRenderer';
import { ConnectedSelectControlRendererEntry } from '../jsonforms/renderers/ConnectedSelectControlRenderer';
import { EnumControlRendererEntry } from '../jsonforms/renderers/EnumControlRenderer';
import { FilterDateControlRendererEntry } from '../jsonforms/renderers/FilterDateControlRenderer';
import { FilterNumberControlRendererEntry } from '../jsonforms/renderers/FilterNumberControlRenderer';
import { InvisibleControlEntry } from '../jsonforms/renderers/InvisibleControlRenderer';
import { LayoutRendererEntry } from '../jsonforms/renderers/LayoutRenderer';
import { LayoutVertical2ColumnsEntry } from '../jsonforms/renderers/LayoutVertical2ColumnsRenderer';
import { StringControlRendererTemplate } from '../jsonforms/renderers/StringControlRenderer';
import { createExpressions } from '../utils/filters';
import { foundationFiltersStyles as styles } from './filters.styles';
import { foundationFiltersTemplate as template } from './filters.template';
/** @internal */
export const filtersLogger = createLogger('foundation-filters');
const stringEntry = {
    renderer: StringControlRendererTemplate,
    tester: rankWith(2, isStringControl),
    mapper: mapStateToControlProps,
};
const fallbackEntry = {
    renderer: StringControlRendererTemplate,
    tester: rankWith(1, isControl),
    mapper: mapStateToControlProps,
};
/** @public */
export const filtersRenderers = [
    fallbackEntry,
    stringEntry,
    BooleanControlEntry,
    LayoutVertical2ColumnsEntry,
    LayoutRendererEntry,
    FilterNumberControlRendererEntry,
    ConnectedMultiselectControlRendererEntry,
    EnumControlRendererEntry,
    ConnectedSelectControlRendererEntry,
    FilterDateControlRendererEntry,
    InvisibleControlEntry,
];
/**
 * Foundation filters component for automatically generated filters based on json schema
 * obtained from the api, supplied initial data or supplied JSON schema.
 * Allowing customisable filters elements using UI schema and set of custom renderers
 * @beta
 */
let Filters = class Filters extends FoundationElement {
    constructor() {
        super(...arguments);
        /**
         * Allows to provide set of renderers used by the filters. If not provided it will default to text-field inputs
         * @public
         */
        this.renderers = filtersRenderers;
        /**
         * Initial data for the filters
         * @public
         */
        this.data = {};
    }
    resourceNameChanged() {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            this.jsonSchema = undefined;
            if (this.resourceName) {
                const jsonSchemaResponse = yield this.connect.getJSONSchema(this.resourceName);
                this.jsonSchema = (_b = (_a = jsonSchemaResponse === null || jsonSchemaResponse === void 0 ? void 0 : jsonSchemaResponse.OUTBOUND) === null || _a === void 0 ? void 0 : _a.$defs) === null || _b === void 0 ? void 0 : _b['global.genesis.message.core.dataserver.QueryRow'];
                (_c = this.jsonSchema) === null || _c === void 0 ? true : delete _c.properties['DETAILS'];
            }
        });
    }
    clearFiltersData() {
        this.data = {};
        this.value = '';
    }
    valueChanged() {
        this.dispatchEvent(new Event('change', { bubbles: true, composed: true }));
    }
    /**
     * @internal
     */
    searchFilters() {
        this.value = '';
        const payload = Object.assign({}, this.data);
        const expressions = createExpressions(payload);
        const criteriaBuilder = new CriteriaBuilder();
        filtersLogger.debug({ payload });
        if (expressions.dateOptions.length > 0 ||
            expressions.numberOptions.length > 0 ||
            expressions.options.length > 0) {
            Object.values(expressions).forEach((value) => {
                if (value.length > 0) {
                    criteriaBuilder.withExpression(value, { join: Join.And() });
                }
            });
            this.value = criteriaBuilder.build();
        }
        this.$emit('submit', { payload });
    }
    /**
     * @internal
     */
    onChange(event) {
        filtersLogger.debug({ event });
        if (event.detail) {
            this.data = event.detail.data;
        }
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        this.data = {};
    }
};
__decorate([
    attr
], Filters.prototype, "resourceName", void 0);
__decorate([
    observable
], Filters.prototype, "uischema", void 0);
__decorate([
    observable
], Filters.prototype, "renderers", void 0);
__decorate([
    observable
], Filters.prototype, "jsonSchema", void 0);
__decorate([
    Connect
], Filters.prototype, "connect", void 0);
__decorate([
    observable
], Filters.prototype, "data", void 0);
__decorate([
    observable
], Filters.prototype, "value", void 0);
Filters = __decorate([
    customElement({
        name: 'foundation-filters',
        template,
        styles,
    })
], Filters);
export { Filters };
